import path from 'path'

import { ConnectionPool, sql } from '@databases/pg'
import sqlite3 from 'better-sqlite3'

export async function makeWanikani(
  db: ConnectionPool,
  srcdb = path.join(__dirname, '../cache/wanikani.db')
) {
  const s3 = sqlite3(srcdb, {
    readonly: true,
  })

  await db.tx(async (db) => {
    const batchSize = 10000

    const lots = s3
      .prepare(
        /* sql */ `
    SELECT "id", "updated_at", "object", "url", "data"
    FROM "subjects"
    `
      )
      .all()

    for (let i = 0; i < lots.length; i += batchSize) {
      console.log(i)
      await db.query(sql`
        INSERT INTO "wanikani.subjects" ("id", "updatedAt", "object", "url", "data")
        VALUES ${sql.join(
          lots
            .slice(i, i + batchSize)
            .map(
              (p) =>
                sql`(${p.id}, ${new Date(p.updated_at)}, ${p.object}, ${
                  p.url
                }, ${JSON.parse(p.data)})`
            ),
          ','
        )}
        ON CONFLICT ("id") DO UPDATE SET
          "updatedAt" = EXCLUDED."updatedAt",
          "object"    = EXCLUDED."object",
          "url"       = EXCLUDED."url",
          "data"      = EXCLUDED."data"
      `)
    }
  })

  s3.close()
}
